import numpy as np
from dlsmicro.backend import analysis_tools
from dlsmicro.backend import io
from dlsmicro.backend import utils
import pandas as pd
import matplotlib.pyplot as plt

def analyze_replicates(csv_name, root_folder, replicates, 
                       T, r, ergodic, Laplace=False, df_save_path=None, 
                       df_file_name=None, save_as_text=True, 
                       save_as_df=True, plot_corr=False, 
                       plot_msd=False, plot_G=False, save_plots=False):

    """ Analyze files exported from Zetasizer software for multiple 
    conditions and plot data per replicate of a condition.

    Parameters
    ----------
    csv_name : str
               Name of every csv file (all the same name)
    root_folder : str
                  Name of folder in which all files to be probed are
    replicates : list of ints
                 List of ints corresponding to the replicate number
    T : float
        Temperature of the experiment in Kelvin
    r : float  
        Radius of particle in experiment in nanometers
    ergodic : boolean
              Ergodicity in experiment
    Laplace : boolean, `optional`
              If `True`, use direct Laplace transform to find
              shear modulus. This is useful because it can
              smooth the data to noise.
    df_save_path : str, `optional` 
                   Path to Dataframe to be saved containing results
                   from DLS microrheology analysis.
                   If `None`, path will be set to root_folder.
    df_file_name : str, `optional`
                   Name of Dataframe to be saved containing results
                   from DLS microrheology analysis.
                   If None, `condition_data.pkl` is default name
    save_as_text : boolean, `optional`
                   If `True`, save each element of Dataframe separately
                   as a text file
    save_as_df : boolean, `optional`
                 If `True`, save the Dataframe
    plot_corr : boolean, `optional`
                If `True`, show plot of the correlation function of
                each replicate
    plot_msd : boolean, `optional`
               If `True`, show plot of the mean-squared displacement 
               of each replicate
    plot_G : boolean, `optional`
             If `True`, show plot of the shear modulus of each replicate
    save_plots : boolean, `optional`
             If `True`, saves plots of correlation function, MSD, G
    """

    if df_save_path == None:
        df_save_path = root_folder
    if df_file_name == None:
        df_file_name = 'replicate_data.pkl'

    # Define scattering vector parameters about solvent
    n = 1.333 # index of refraction of water (default)
    theta = 173.*np.pi/180. 
    lam = 633.

    ####################################################
    # Analyze data
    # Don't edit this unless you know what you're doing
    ####################################################

    # Create pandas dataframe to organize replicate data
    df = pd.DataFrame(columns=['time', 'MSD', 'alpha', 'omega', 'G1', 'G2',
                               'replicate', 'scattering','epos'])
    for replicate in replicates:
        file_path = '%s/replicate%s/%s' % (root_folder, replicate, csv_name)

        # Read the data and truncate over the desired time windows
        data_dict = io.read_zetasizer_csv_to_dict(file_path, 0)
        [t, g, I, Ie, point_pos, epos] = list(data_dict.values())

        # Figure out where data is no longer trustworthy (correlation function goes to zero)
        tinds = [3, len(g)]
        tinds[1] = np.argmax(g<0.05)
        if tinds[1] == 0:
            tinds[1] = len(g) - 1
        g = g[tinds[0]:tinds[1]]
        t = t[tinds[0]:tinds[1]]

        # Get a DLS microrheology object that contains all raw data
        # and analyzed results. 
        q = analysis_tools.calc_q(n, theta, lam)
        dlsmicro_df = analysis_tools.full_dlsur_analysis(t, g, ergodic, r, T, q,
                                                         I, Ie)

        # Store the scattering vs. position data
        scattering = np.zeros(len(dlsmicro_df['t']))
        positions = np.zeros_like(scattering)
        scattering[0:len(Ie)] = Ie
        positions[0:len(epos)] = epos

        # Laplace transformed modulus
        if Laplace:
            [omega_L, G1_L, G2_L] = analysis_tools.shear_modulus_laplace_transform(t, 
                                                   dlsmicro_df['msd_smooth'], r, T)
            dlsmicro_df['G1'], dlsmicro_df['G2'] = utils.laplace_merge(dlsmicro_df['t'], 
                                                                     dlsmicro_df['G1'], 
                                                                     dlsmicro_df['G2'],
                                                                     G1_L,G2_L)

        # Construct pandas dataframe for this replicate and append it to the
        # master dataframe
        dlsmicro_df['replicate'] = [replicate]*len(dlsmicro_df['t'])
        dlsmicro_df['scattering'] = scattering
        dlsmicro_df['epos'] = positions

        df = pd.concat((df,dlsmicro_df), axis=0, sort=True)

        ###############################################
        # Plot the analyzed data for this replicate
        ###############################################

        if plot_corr:
            plt.plot(t, g, '-r')
            plt.xscale('log')
            plt.xlabel('$\\mathregular{time\ (\\mu s)}$')
            plt.ylabel('g')
            if save_plots:
              save_path = '%s/replicate%s/corr' % (root_folder,
                                                   replicate)
              plt.savefig(save_path)
            plt.show()
        if plot_msd:
            plt.plot(dlsmicro_df['t'], dlsmicro_df['msd_smooth'], '-r')
            plt.xscale('log')
            plt.yscale('log')
            plt.xlabel('$\\mathregular{time\ (\\mu s)}$')
            plt.ylabel('MSD')
            if save_plots:
              save_path = '%s/replicate%s/msd' % (root_folder,
                                                   replicate)
              plt.savefig(save_path)
            plt.show()
        if plot_G:
            plt.plot(dlsmicro_df['omega'], dlsmicro_df['G1'], '-r')
            plt.plot(dlsmicro_df['omega'], dlsmicro_df['G2'], '--r')
            plt.ylabel('$\\mathregular{G^*\ (Pa)}$')
            plt.xlabel('$\\mathregular{\\omega\ (s^{-1})}$')
            plt.legend(['$\\mathregular{G^{\\prime}}$',
                        '$\\mathregular{G^{\\prime \\prime}}$'], frameon=False,)
            plt.xscale('log')
            plt.yscale('log')
            if save_plots:
              save_path = '%s/replicate%s/G' % (root_folder,
                                                   replicate)
              plt.savefig(save_path)
            plt.show()

        ##############################################
        # Save analysis results
        ##############################################

        if save_as_text:
            save_path = '%s/replicate%s/' % (root_folder, replicate)
            for i in dlsmicro_df.columns:
                np.savetxt('%s/%s' % (save_path,i), dlsmicro_df[i].values)


    #################################################
    # Save the pandas dataframe
    #################################################
    save_path = df_save_path + '/' + df_file_name
    df.to_pickle(save_path)



import numpy as np
from dlsmicro.backend import analysis_tools
from dlsmicro.backend import io
from dlsmicro.backend import utils
import pandas as pd
import matplotlib.pyplot as plt

def analyze_conditions(csv_name, root_folder, condition_dir, 
                       replicate_dict, T, r, erg, Laplace=False, 
                       df_save_path=None, df_file_name=None, 
                       save_as_text=True, save_as_df=True,
                       plot_corr=False, plot_msd=False, plot_G=False,
                       save_plots=False):

    """ Analyze files exported from Zetasizer software for multiple 
    conditions and plot data per replicate of a condition.

    Parameters
    ----------
    csv_name : str
               Name of every csv file (all the same name)
    root_folder : str
                  Name of folder in which all files to be probed are
    condition_dir : dictionary
                   Dictionary of conditions and respective folders
    replicate_dict : dictionary
                     Dictionary of replicates for each condition
    T : float or dictionary of float
        Temperature of the experiment/condition in Kelvin
    r : float or dictionary of float 
        Radius of particle in experiment/condition in nanometers
    erg : boolean or dictionary of boolean
          Ergodicity in experiment/condition
    Laplace : boolean, `optional`
              If `True`, use direct Laplace transform to find
              shear modulus. This is useful because it can
              smooth the data to noise.
    df_save_path : str, `optional` 
                   Path to Dataframe to be saved containing results
                   from DLS microrheology analysis.
                   If `None`, path will be set to root_folder.
    df_file_name : str, `optional`
                   Name of Dataframe to be saved containing results
                   from DLS microrheology analysis.
                   If None, `condition_data.pkl` is default name
    save_as_text : boolean, `optional`
                   If `True`, save each element of Dataframe separately
                   as a text file
    save_as_df : boolean, `optional`
                 If `True`, save the Dataframe
    plot_corr : boolean, `optional`
                If `True`, show plot of the correlation function of
                each replicate
    plot_msd : boolean, `optional`
               If `True`, show plot of the mean-squared displacement 
               of each replicate
    plot_G : boolean, `optional`
             If `True`, show plot of the shear modulus of each replicate
    save_plots : boolean, `optional`
             If `True`, saves plots of correlation function, MSD, G
    """

    conditions = list(condition_dir.keys())
    if type(T) is dict:
        if T.keys() != condition_dir.keys():
            raise Exception('Keys for temp dictionary do not match conditions')
        T_dict = T
    else:
        T_dict = dict((k, T) for k, v in condition_dir.items())
    if type(r) is dict:
        if r.keys() != condition_dir.keys():
            raise Exception('Keys for radius dictionary do not match conditions')
        r_dict = r
    else:
        r_dict = dict((k, r) for k, v in condition_dir.items())
    if type(erg) is dict:
        if erg.keys() != condition_dir.keys():
            raise Exception('Keys for ergodicity dictionary do not match conditions')
        erg_dict = erg
    else:
        erg_dict = dict((k, erg) for k, v in condition_dir.items())
    if df_save_path == None:
        df_save_path = root_folder
    if df_file_name == None:
        df_file_name = 'condition_data.pkl'

    # Define scattering vector parameters about solvent
    n = 1.333 # index of refraction of water (default)
    theta = 173.*np.pi/180. 
    lam = 633.

    ####################################################
    # Analyze data
    # Don't edit this unless you know what you're doing
    ####################################################

    # Create pandas dataframe to organize replicate data
    df = pd.DataFrame(columns=['t', 'msd_smooth', 'alpha', 'omega', 'G1', 'G2',
                               'replicate', 'condition', 'id', 'scattering', 'epos'])
    idx = 0
    for condition in conditions:
        for replicate in replicate_dict[condition]:
            file_path = '%s/%s/replicate%s/%s' % (root_folder,
                                                  condition_dir[condition],
                                                  replicate, csv_name)

            # Read the data and truncate over the desired time windows
            data_dict = io.read_zetasizer_csv_to_dict(file_path, 0)
            [t, g, I, Ie, point_pos, epos] = list(data_dict.values())

            # Figure out where data is no longer trustworthy (correlation function goes to zero)
            tinds = [3, len(g)]
            tinds[1] = np.argmax(g<0.05)
            if tinds[1] == 0:
                tinds[1] = len(g) - 1
            g = g[tinds[0]:tinds[1]]
            t = t[tinds[0]:tinds[1]]

            # Get a DLS microrheology object that contains all raw data
            # and analyzed results. 
            T = T_dict[condition]
            r = r_dict[condition]
            ergodic = erg_dict[condition]
            q = analysis_tools.calc_q(n, theta, lam)
            dlsmicro_df = analysis_tools.full_dlsur_analysis(t, g, ergodic, r, T, q,
                                                             I, Ie)

            # Store the scattering vs. position data
            scattering = np.zeros(len(dlsmicro_df['t']))
            positions = np.zeros_like(scattering)
            scattering[0:len(Ie)] = Ie
            positions[0:len(epos)] = epos

            # Laplace transformed modulus
            if Laplace:
                [omega_L, G1_L, G2_L] = analysis_tools.shear_modulus_laplace_transform(t, 
                                                       dlsmicro_df['msd_smooth'], r, T)
                dlsmicro_df['G1'], dlsmicro_df['G2'] = utils.laplace_merge(dlsmicro_df['t'], 
                                                                         dlsmicro_df['G1'], 
                                                                         dlsmicro_df['G2'],
                                                                         G1_L,G2_L)

            # Construct pandas dataframe for this replicate and append it to the
            # master dataframe
            dlsmicro_df['replicate'] = [replicate]*len(dlsmicro_df['t'])
            dlsmicro_df['scattering'] = scattering
            dlsmicro_df['epos'] = positions
            dlsmicro_df['condition'] = condition
            dlsmicro_df['id'] = idx

            idx += 1
            df = pd.concat((df,dlsmicro_df), axis=0, sort=True)

            ###############################################
            # Plot the analyzed data for this replicate
            ###############################################

            if plot_corr:
                plt.plot(t, g, '-r')
                plt.xscale('log')
                plt.xlabel('$\\mathregular{time\ (\\mu s)}$')
                plt.ylabel('g')
                plt.tight_layout()
                if save_plots:
                    save_path = '%s/%s/replicate%s/corr' % (root_folder,
                                                            condition_dir[condition],
                                                            replicate)
                    plt.savefig(save_path)
                plt.show()
            if plot_msd:
                plt.plot(dlsmicro_df['t'], dlsmicro_df['msd_smooth'], '-r')
                plt.xscale('log')
                plt.yscale('log')
                plt.xlabel('$\\mathregular{time\ (\\mu s)}$')
                plt.ylabel('MSD')
                plt.tight_layout()
                if save_plots:
                    save_path = '%s/%s/replicate%s/msd' % (root_folder,
                                                            condition_dir[condition],
                                                            replicate)
                    plt.savefig(save_path)
                plt.show()
            if plot_G:
                plt.plot(dlsmicro_df['omega'], dlsmicro_df['G1'], '-r')
                plt.plot(dlsmicro_df['omega'], dlsmicro_df['G2'], '--r')
                plt.ylabel('$\\mathregular{G^*\ (Pa)}$')
                plt.xlabel('$\\mathregular{\\omega\ (s^{-1})}$')
                plt.legend(['$\\mathregular{G^{\\prime}}$',
                            '$\\mathregular{G^{\\prime \\prime}}$'], frameon=False,)
                plt.xscale('log')
                plt.yscale('log')
                plt.tight_layout()
                if save_plots:
                    save_path = '%s/%s/replicate%s/G' % (root_folder,
                                                            condition_dir[condition],
                                                            replicate)
                    plt.savefig(save_path)
                plt.show()

            ##############################################
            # Save analysis results
            ##############################################
            if save_as_text:
                save_path = '%s/%s/replicate%s/' % (root_folder,
                                                    condition_dir[condition],
                                                    replicate)
                for i in dlsmicro_df.columns[:-2]:
                    np.savetxt('%s/%s' % (save_path,i), dlsmicro_df[i].values)

    #################################################
    # Save the pandas dataframe
    #################################################
    if save_as_df:
        save_path = df_save_path + '/' + df_file_name
        df.to_pickle(save_path)



import numpy as np


# stretched exponential decay function
def stretched_exp(x, x0, a, beta):
    return x0*np.exp(-a*(x**beta))


# 'exp exp' decay function
def expexp(x, a0, a1, lam, beta):
    return a0*np.exp(-a1*(1-np.exp(-lam*x**beta)))



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from dlsmicro.backend import plot_tools
from matplotlib import rc
import matplotlib as mpl

def plot_replicates(df_path, replicates, replic_color=None,
					plot_ci=True, plot_G_replicates=True,
					plot_alpha_replicates=True, plot_scattering=False,
					add_scaling=False, scaling_frac=None):

	""" Plot DLS microrheology output data for
    multiple replicates after analysis and saving as dataframe.

    Parameters
    ----------
    df_path : str
    		  Path to saved dataframe to plot
    replicates : list of ints
                 List of ints corresponding to the replicate number
    replic_color : dictionary, `optional`
    			 Dictionary of replicates and respective color
    			 If `None`, colors are default colors. This 
    			 applies to the scattering plot. Otherwise, color
    			 plotted for G and alpha is first color in dictionary
    plot_ci : boolean, `optional`
    		  If `True`, plot error bars around G and alpha
    plot_G_replicates : boolean, `optional`
    					If `True`, plot G for all conditions averaged
    					over all replicates
    plot_alpha_replicates : boolean, `optional`
    						If `True`, plot alpha for all conditions
    						averaged over all replicates
    plot_scattering : boolean, `optional`
    				  If `True`, plot scattering for all conditions
    add_scaling : boolean, `optional`
    			  If `True`, add scaling defined by `scaling_frac` to plot
    scaling_frac : list of float, `optional`
    			   List of 2 floats, where the first number is numerator 
    			   of fraction and second is denominator of fraction
    """

	df = pd.read_pickle(df_path)

	# set colors and labels in dictionaries
	if replic_color == None:
		cmap = plt.cm.get_cmap('hot',len(replicates)*2)
		colors = [cmap(i) for i in range(len(replicates))]
		replic_color = dict(zip(replicates, colors))

	if plot_G_replicates:
		# Plot style options
		rc('axes', labelsize=24.)
		rc('xtick', labelsize=18.)
		rc('ytick', labelsize=18.)
		rc('lines', markersize=10)
		rc('lines', linewidth=3)

		fig, ax1 = plt.subplots(1, 1)
		plot_tools.plot_replicates_from_df(df, 'G1', plot_ci=plot_ci, 
										   color=replic_color[replicates[0]])
		plot_tools.plot_replicates_from_df(df, 'G2', plot_ci=plot_ci, 
									  	   color=replic_color[replicates[0]], ls='--')

		if add_scaling:
			plot_tools.add_w_scaling(df[df['replicate'] == replicates[0]]['omega'][0:50],
				          			 scaling_frac,
				          		     df[df['replicate'] == replicates[0]]['G2'][2],
				          			 [2./6.,3./6.])
		handles = [None] * (2)
		handles[0] = plt.Line2D((0, 1), (0, 0), color=replic_color[replicates[0]], dashes='',
								label='$\mathregular{G^{\prime}}$')
		handles[1] = plt.Line2D((0, 1), (0, 0), color=replic_color[replicates[0]], ls='--',
								label='$\mathregular{G^{\prime \prime}}$')
		plt.legend(handles=handles, frameon=False, fontsize=16)
		plt.ylabel('$\\mathregular{G^*\ (Pa)}$')
		plt.xlabel('$\\mathregular{\\omega\ (s^{-1})}$')
		plt.xscale('log')
		plt.yscale('log')

		# Force plot to show ticks at every decade and minor ticks
		minor_tick_marks = [0.1*x for x in range(1, 10)]
		locmax = mpl.ticker.LogLocator(base=10, numticks=12)
		locmin = mpl.ticker.LogLocator(base=10, numticks=12, subs=minor_tick_marks)
		ax1.xaxis.set_major_locator(locmax)
		ax1.xaxis.set_minor_locator(locmin)
		ax1.xaxis.set_minor_formatter(mpl.ticker.NullFormatter())
		plt.tight_layout()
		plt.show()

	if plot_alpha_replicates:
		# Plot style options
		rc('axes', labelsize=24.)
		rc('xtick', labelsize=18.)
		rc('ytick', labelsize=18.)
		rc('lines', markersize=10)
		rc('lines', linewidth=3)

		fig, ax1 = plt.subplots(1, 1)
		plot_tools.plot_replicates_from_df(df, 'alpha', plot_ci=plot_ci, 
			                               color=replic_color[replicates[0]])
		plt.ylabel('$\\mathregular{\\alpha}$')
		plt.xlabel('$\\mathregular{\\omega\ (s^{-1})}$')
		plt.xscale('log')
		plt.tight_layout()
		plt.show()

	if plot_scattering:
		# Plot style options
		rc('axes', labelsize=18.)
		rc('xtick', labelsize=14.)
		rc('ytick', labelsize=14.)
		rc('lines', markersize=10)
		rc('lines', linewidth=3)

		fig, ax1 = plt.subplots(1, 1)

		for replicate in replicates:
			epos = df[df['replicate'] == replicate]['epos'].values
			scattering = df[df['replicate'] == replicate]['scattering'].values
			plot_over = np.argwhere(epos)
			plt.plot(epos[plot_over],scattering[plot_over],
				     color=replic_color[replicate],linestyle='None',marker='.')

		# Define line objects for legend so that line color is black rather than
		# inheriting the color of one of the conditions
		handles = [None] * len(replicates)
		for replicate in range(len(replicates)):
			handles[replicate] = plt.Line2D((0, 1), (0, 0), color=replic_color[replicates[replicate]],
											ls='None', marker='o', label=replicates[replicate])
		plt.legend(handles=handles, frameon=False, fontsize=10, handlelength=2.0)
		plt.ylabel('Scattering Intensity')
		plt.xlabel('Position')
		plt.yscale('log')
		plt.tight_layout()
		plt.show()



import numpy as np
from dlsmicro.backend import analysis_tools
from dlsmicro.backend import io
from dlsmicro.backend import utils
import matplotlib.pyplot as plt
import pandas as pd

def analyze_time_points(file_path, T, r, ergodic, n_points, n_positions,
                        Laplace=False, df_save_path=None, df_file_name=None,
                        save_as_txt=True, save_as_df=True, 
                        plot_corr=False, plot_msd=False, plot_G=False):

    """ Analyze files exported from Zetasizer software for time-
    dependent measurements and plot data per time point.

    Parameters
    ----------
    file_path : str
               Path to exported csv file
    T : float
        Temperature of the experiment in Kelvin
    r : float  
        Radius of particle in experiment in nanometers
    ergodic : boolean
              Ergodicity in experiment
    n_points : int
               Corresponds to the number of measurements taken over 
               duration of experiment
    n_positions : int
                  Corresponds to the number of measurements taken
                  over different positions at end of experiment
    Laplace : boolean, `optional`
              If `True`, use direct Laplace transform to find
              shear modulus. This is useful because it can
              smooth the data to noise.
    df_save_path : str, `optional` 
                   Path to Dataframe to be saved containing results
                   from DLS microrheology analysis.
                   If `None`, path will be set to file_path.
    df_file_name : str, `optional`
                   Name of Dataframe to be saved containing results
                   from DLS microrheology analysis.
                   If None, `condition_data.pkl` is default name
    save_as_text : boolean, `optional`
                   If `True`, save each element of Dataframe separately
                   as a text file
    save_as_df : boolean, `optional`
                 If `True`, save the Dataframe
    plot_corr : boolean, `optional`
                If `True`, show plot of the correlation function of
                each replicate
    plot_msd : boolean, `optional`
               If `True`, show plot of the mean-squared displacement 
               of each replicate
    plot_G : boolean, `optional`
             If `True`, show plot of the shear modulus of each replicate
    """

    if df_save_path == None:
        files = file_path.split('/')
        file_name = files[-1]
        df_save_path = file_path[:-(len(file_name))]
    if df_file_name == None:
        df_file_name = 'time_course.pkl'

    # Row numbers for scattering intensity sweep (int_rcds)
    # Row numbers for correlation time points (time_points)
    int_rcds = range(n_points + 1, n_points + n_positions)
    time_points = range(1, n_points, 1)
    # if cuvette == 'disposable':
    #     int_rcds = range(13, 33)
    #     time_points = range(1, 12, 1)
    # elif cuvette == 'quartz':
    #     int_rcds = range(19, 51)
    #     time_points = range(1, 18, 1)
    # else:
    #     raise Exception('cuvette type entered is not valid')

    # Define scattering vector parameters about solvent
    n = 1.333 # index of refraction of water (default)
    theta = 173.*np.pi/180. 
    lam = 633.

    # Create pandas dataframe to organize time point data
    df = pd.DataFrame(columns=['time', 'MSD', 'alpha', 'omega', 'G1', 'G2',
                               'time_point', 'scattering','epos'])

    for tp in time_points:
        save_suffix = 'time_point_%s.txt' % tp
        ####################################################
        # Analyze data
        # Don't edit this unless you know what you're doing
        ####################################################

        # Read the data and truncate over the desired time windows
        data_dict = io.read_zetasizer_csv_to_dict(file_path, tp, intensities_rows=int_rcds)
        [t, g, I, Ie, point_pos, epos] = list(data_dict.values())

        # Figure out where data is no longer trustworthy (correlation function goes to zero)
        tinds = [3, len(g)]
        tinds[1] = np.argmax(g<0.05)
        if tinds[1] == 0:
            tinds[1] = len(g) - 1
        g = g[tinds[0]:tinds[1]]
        t = t[tinds[0]:tinds[1]]

        # Get a DLS microrheology object that contains all raw data
        # and analyzed results. 
        q = analysis_tools.calc_q(n, theta, lam)
        dlsmicro_df = analysis_tools.full_dlsur_analysis(t, g, ergodic, r, T, q,
                                                         I, Ie)

        # Store the scattering vs. position data
        scattering = np.zeros(len(dlsmicro_df['t']))
        positions = np.zeros_like(scattering)
        scattering[0:len(Ie)] = Ie
        positions[0:len(epos)] = epos

        # Laplace transformed modulus
        if Laplace:
            [omega_L, G1_L, G2_L] = analysis_tools.shear_modulus_laplace_transform(t, 
                                                   dlsmicro_df['msd_smooth'], r, T)
            dlsmicro_df['G1'], dlsmicro_df['G2'] = utils.laplace_merge(dlsmicro_df['t'], 
                                                                     dlsmicro_df['G1'], 
                                                                     dlsmicro_df['G2'],
                                                                     G1_L,G2_L)

        # Construct pandas dataframe for this replicate and append it to the
        # master dataframe
        dlsmicro_df['time_point'] = [tp]*len(dlsmicro_df['t'])
        dlsmicro_df['scattering'] = scattering
        dlsmicro_df['epos'] = positions

        df = pd.concat((df,dlsmicro_df), axis=0, sort=True)

        ###############################################
        # Plot the analyzed data
        ###############################################

        if plot_corr:
            plt.plot(t, g, '-r')
            plt.xscale('log')
            plt.xlabel('$\\mathregular{time\ (\\mu s)}$')
            plt.ylabel('g')
            plt.show()
        if plot_msd:
            plt.plot(dlsmicro_df['t'], dlsmicro_df['msd_smooth'], '-r')
            plt.xscale('log')
            plt.yscale('log')
            plt.xlabel('$\\mathregular{time\ (\\mu s)}$')
            plt.ylabel('MSD')
            plt.show()
        if plot_G:
            plt.plot(dlsmicro_df['omega'], dlsmicro_df['G1'], '-r', lw=2.0)
            plt.plot(dlsmicro_df['omega'], dlsmicro_df['G2'], '--r', lw=2.0)
            plt.ylabel('$\\mathregular{G^*\ (Pa)}$')
            plt.xlabel('$\\mathregular{\\omega\ (s^{-1})}$')
            plt.legend(['$\\mathregular{G^{\\prime}}$',
                        '$\\mathregular{G^{\\prime \\prime}}$'], frameon=False,)
            plt.xscale('log')
            plt.yscale('log')
            plt.show()

        ##############################################
        # Save analysis results
        ##############################################

        if save_as_txt:
            for i in dlsmicro_df.columns:
                np.savetxt('%s/%s' % (df_save_path,i), dlsmicro_df[i].values)

    #################################################
    # Save the pandas dataframe
    #################################################
    if save_as_df:
        save_path = df_save_path + '/' + df_file_name
        df.to_pickle(save_path)



import numpy as np
import matplotlib.pyplot as plt
from matplotlib import rc
from dlsmicro.backend import plot_tools
import pandas as pd

def plot_time_points(df_path, n_points, plot_G_replicates=True,
					plot_MSD_replicates=True, plot_scattering=False, 
					add_scaling=False, scaling_frac=None):

	""" Plot DLS microrheology output data for time-dependent 
	measurements after analysis and saving as dataframe.

    Parameters
    ----------
    df_path : str
    		  Path to saved dataframe to plot
    n_points : int
               Corresponds to the number of measurements taken over 
               duration of experiment
    plot_G_replicates : boolean, `optional`
    					If `True`, plot shear modulus for all time points
    plot_MSD_replicates : boolean, `optional`
    					  If `True`, plot mean-squared displacement
    					  for all time points
    plot_scattering : boolean, `optional`
    				  If `True`, plot scattering for all time points
    add_scaling : boolean, `optional`
    			  If `True`, add scaling defined by `scaling_frac` to plot
    scaling_frac : list of float, `optional`
    			   List of 2 floats, where the first number is numerator 
    			   of fraction and second is denominator of fraction
    """

	# Plot style options
	rc('axes', labelsize=20.)
	rc('xtick', labelsize=16.)
	rc('ytick', labelsize=16.)
	rc('lines', markersize=10)
	rc('lines', linewidth=1)

	# Row numbers for correlation time points (time_points)
	time_points = range(1, n_points, 1)
	# if cuvette == 'disposable':
	# 	time_points = range(1, 12, 1)
	# elif cuvette == 'quartz':
	# 	time_points = range(1, 18, 1)
	# else:
	# 	raise Exception('cuvette type entered is not valid')

	cmap = plt.cm.get_cmap('viridis',len(time_points))
	colors = [cmap(i) for i in range(len(time_points))]

	df = pd.read_pickle(df_path)

	if plot_MSD_replicates:
		for i, tp in enumerate(time_points):
			dfi = df[df['time_point'] == tp]
			time = dfi['t']
			msd = dfi['msd_smooth']
			plt.plot(time*1.e6, msd, color=colors[i], ls='-')
		plt.legend(time_points, frameon=False)
		plt.xscale('log')
		plt.yscale('log')
		plt.xlabel('$\\mathregular{\\tau\ (s)}$')
		plt.ylabel('$\\mathregular{MSD\ (nm^2)}$')
		plt.tight_layout()
		plt.show()

	if plot_G_replicates:
		for i, tp in enumerate(time_points):
			dfi = df[df['time_point'] == tp]
			freq = dfi['omega']
			G1 = dfi['G1']
			G2 = dfi['G2']
			plt.plot(freq, G1, color=colors[i], ls='-')
			plt.plot(freq, G2, color=colors[i], ls='--')
		if add_scaling:
			plot_tools.add_w_scaling(df[df['time_point'] == time_points[0]]['omega'][0:50],
						  scaling_frac,
						  df[df['time_point'] == time_points[0]]['G2'][2],
						  [2./6.,3./6.])
		plt.legend(time_points, frameon=False)
		plt.xscale('log')
		plt.yscale('log')
		plt.xlabel('$\\mathregular{\\omega\ (s^{-1})}$')
		plt.ylabel('$\\mathregular{G^*\ (Pa)}$')
		plt.tight_layout()
		plt.show()

	if plot_scattering:
		dfi = df[df['time_point'] == time_points[0]]
		epos = dfi['epos'].values
		scattering = dfi['scattering'].values
		plot_over = np.argwhere(epos)
		plt.plot(epos[plot_over],scattering[plot_over],linestyle='None',marker='.')
		plt.ylabel('Scattering Intensity')
		plt.xlabel('Position')
		plt.yscale('log')
		plt.tight_layout()
		plt.show()




""" Module for parsing data exported from Zetasizer software"""
import pandas as pd
import numpy as np

# Columns name order for the dlsmicro_export.edf template
default_column_order = ['Record', 'Sample Name', 'Measurement Position',
                        'Correlation Data', 'Correlation Delay Times',
                        'Distribution Fit Data',
                        'Distribution Fit Delay Times',
                        'Cumulants Fit Data', 'Cumulants Fit Delay Times',
                        'Derived Count Rate', 'Measured Intercept',
                        'Measured Baseline']


def read_zetasizer_csv_to_dict(file_path, row,
                               intensities_rows=None,
                               column_order=default_column_order,
                               use_zetasizer_g1=True):
    """ Read csv file exported from the Zetasizer software to a
    dictionary containing data relevant to DLS microrheology analysis

    Parameters
    ----------
    file_path : str
                Path to the .csv file to be read
    row : int
          Row number (0-indexed) for the measurement record containing the
          correlation data
    intensities_rows : list of int, `optional`
                       List of rows (0-indexed) corresponding to the
                       scattering intensity measurements for the
                       broken-ergodicity
                       correction. If ``None``, it is assumed that the
                       intensity measurements
                       begin after the row number for the correlation data and
                       end at the last
                       row of the .csv file.
    column_order : list of str, `optional`
                   Ordered list names for the columns in the .csv file
                   (depends on your export template). If you use
                   the `dlsmicro_export.edf`, this parameter is not necessary.
                   See ``dlsmicro.io.default_column_order``
    use_zetasizer_g1 : boolean, `optional`
                       If `True`, the estimated intermediate scattering
                       function `g1`, measured
                       baseline, and measured intercept exported by the
                       Zetasizer software are used to calculate the correlation
                       function. This is useful because `g1` is exported at
                       higher numerical
                       precision than the correlation function.
                       The `g1` exported from the
                       Zetasizer is `not` the true `g1` for non-ergodic
                       samples, but this
                       option correctly inverts the formula used by
                        the Zetasizer.

    Returns
    -------
    data_dict : dictionary
                Python dictionary containing the keys below
    'time_lag' : 1d-array
                 Vector of time-lags (in microseconds) at which the
                 correlation function is measured.
    'correlation' : 1d-array
                    Vector of values of the correlation coefficient at the
                    time-lags
                    ``data_dict['time_lag']``
    'point_intensity' : float
                        Scattering intensity at the measurement position where
                        ``data_dict['correlation']`` is collected
    'ensemble_intensity' : 1d-array
    'point_position' : float
                       Meausrement position in the cuvette (in mm) where the
                       data for the
                       correlation function ``data_dict['correlation']``
                       is collected
    'ensemble_positions' : 1-d array
                           Vector of measurement positions in the cuvette
                           corresponding to
                           the scattering intensities in
                           ``data_dict['ensemble_intensity']``

    Notes
    -----
    This function will correctly parse the .csv file generated using the
    dlsmicro_export.edf Zetasizer software template. If you would like to use
    this function for parsing .csv files
    from a different user-generated Zetasizer template, 
    the following parameters `must`
    be exported. In addition,
    a list of the paramters in the order in which they occur in the .csv
    columns must be supplied. Note that
    if generating your own Zetasizer template, you should not use the "include
    headers" option. This does not work
    well for exporting the correlation data.

    'Correlation Data' : Zetasizer exports ss a string of comma separated
                         values, e.g. "1.000, 0.987, 0.921, ..."
    'Correlation Data Delay Times' : Zetasizer exports as a string of comma
                                     separated values, e.g. "0.50, 1.0, 1.5, ..."
    'Derived Count Rate'
    'Measurement Position'

    If ``use_zetsizer_g1==True``, then the export template ``must`` also
    include the following:
    'Distribution Fit Data': Zetasizer exports as a string of comma separated
     values, e.g. "0.50, 1.0, 1.5, ..."
    'Distribution Fit Delay Times': Zetasizer exports as a string of comma
                                    separated values,
                                    e.g. "0.50, 1.0, 1.5, ..."
    'Measured Intercept'
    'Measured Baseline'
    """
    # Read csv to pandas dataframe
    df = pd.read_csv(file_path, header=None, names=column_order)
    # By default, assume that scattering intensity measurements for
    # broken ergodicity correction are in second row until the end of the file
    if intensities_rows is None:
        intensities_rows = range(row + 1, len(df))

    g = np.array(
        [float(i) for i in df.iloc[row]['Correlation Data'].split(',')])
    t = np.array(
        [float(i) for i in df.iloc[row]['Correlation Delay Times'].split(',')])
    # Get the g1 correlation function data from zetasizer, which is
    # more precise than g2
    tfit = np.array(
        [float(i) for i in df.iloc[row]
         ['Distribution Fit Delay Times'].split(',')])
    g1fit = np.array(
        [float(i) for i in df.iloc[row]
         ['Distribution Fit Data'].split(',')])

    B = df.iloc[row]['Measured Baseline']
    # Get scattering intensity for the row of interest and
    # the ensemble
    Ie = df.iloc[intensities_rows]['Derived Count Rate']
    Ip = df.iloc[row]['Derived Count Rate']

    point_pos = df.iloc[row]['Measurement Position']
    epos = df.iloc[intensities_rows]['Measurement Position']
    g = np.copy(g)

    # Replace g with the data obtained from the g1 correlation
    # function where the data exists
    if use_zetasizer_g1:
        gadj = B + g1fit**2.
        tinds = [np.argmin(np.abs(t-ti)) for ti in tfit]
        g[tinds] = gadj

    data_dict = {'time_lag': t, 'correlation': g, 'point_intensity': Ip,
                 'ensemble_intensities': Ie, 'point_position': point_pos,
                 'ensemble_positions': epos}
    return data_dict



import numpy as np
from numpy import linalg
from numpy import random
from scipy import integrate
from scipy.optimize import curve_fit

##########################################################
#gauusian_weight
#compute a gaussian weight matrix W, given a vector x
#the reference observation point i (index) and a bandwith
#tau
##########################################################

def gaussian_weight(x, i, tau):
    #compute the vector of weights
    w = np.exp(-((x-x[i])**2.)/(tau))
    #transform into a diagonal array
    W = np.diag(w)
    return W

#########################################################
#Weighted linaer regression
#perform linear regression given a matrix of inputs
#X, whose rows correspond to observations and whose columns
#correspond to features and a vector of outputs
#y
#inputs:
#X - matrix of inputs (mXn) with m features and n observations
#y - vector of ouput observations (size n)
#W - n X n weight matrix
#ouputs:
#theta - vector of best-fit parameters
#########################################################

def weighted_linear_reg(X,y,W):
    m1 = linalg.inv(np.dot(np.dot(X.T,W),X))
    m2 = np.dot(np.dot(X.T,W),y)

    theta = np.dot(m1,m2)
    return theta

#####################################################################
#Loess
#Compute a locally weighted linear regression given a vector
#of inputs x and a vector of outputs y
#Inputs
#x - vector of inputs
#y - vector of ouputs
#degree - degree of polynomial for fit (1 or 2)
#alpha - smoothing parameter. Defined as the ratio of tau**2. divided
#by the square in the difference between the two extrema in x


def loess(x,y,degree,alpha):
    n = len(x)
    #######################################################
    #First degree polynomial regression
    #######################################################
    if degree == 1:

        #First feature in the X array is a constant
        #Second feature is the value of x
        const = np.ones(n)
        X = np.array([const,x]).T
        W = np.identity(n)
        tau = alpha * np.sqrt((x[0]-x[-1])**2.)

        #Construct the theta matrix
        Theta = np.zeros([2,n])
        #Construct the prediction vector
        yp = np.zeros(n)
        #Loop over observations
        for i in range(n):
            #compute the weight matrix
            W = gaussian_weight(x,i,tau)
           #perform linear regression
            theta = weighted_linear_reg(X,y,W)
            Theta[:,i] = theta
            yp[i] = np.dot(X[i,:],theta)
        return [Theta,yp]

    #######################################################
    #Second degree polynomial regression
    #######################################################
    if degree == 2:

        #First feature in the X array is a constant
        #Second feature is the value of x
        #Third feature is the value of x**2.
        const = np.ones(n)
        X = np.array([const,x,x**2.]).T
        W = np.identity(n)
        tau = alpha * np.sqrt((x[0]-x[-1])**2.)

        #Construct the theta matrix
        Theta = np.zeros([3,n])

        #Construct the prediction vector
        yp = np.zeros(n)
        #Loop over observations
        for i in range(n):
            #compute the weight matrix
            W = gaussian_weight(x,i,tau)
            #perform linear regression
            theta = weighted_linear_reg(X,y,W)
            Theta[:,i] = theta
            yp[i] = np.dot(X[i,:],theta)
        return [Theta,yp]

##################################
#Numerical Laplace transform
##################################
#Inputs
#t - time vector
#f - vector of discrete values of function to be transformed
#S - vector of Laplace frequencies to evaluate the transform at


def laplace(t, f, S):
    L = np.zeros(len(S))
    for i, s in enumerate(S):
        y = f*np.exp(-s*t)
        L[i] = integrate.trapz(y, t)
    return L


def get_cross_validation_score(t, y, func, p0=None):
    """ Obtain the leave-one-out cross-validation score for
    a functional model of a data set over a given fitting window.

    Parameters
    ----------
    t : 1-d array
        Length N vector  of values for the independent variable of a dataset.
    y : 1-d array
        Length N vector of values for the dependent variable of a dataset.
    func : callable function
           Model to fit the data against. Must be of the form
           `f(t, p1, p2, ..., pM)` where `t` is the independent variable and
           `p1, p2, ..., pM` is a set of M parameters to fit.

    p0 : 1-d array or list, `optional`
         Initial guesses for the M parameters to fit, [p1, p2, ..., pM]

    Returns
    -------
    cv : float 
         Leave-one-out cross-validation score for the model
    """
    cv = 0.
    yskip = y
    tskip = t
    n = len(tskip)
    # Loop over each (t, y) point and perform a
    # fit to func with that point removed
    # calculate MSE for that point
    for i in range(n):
        ytest = np.delete(yskip, i)
        ttest = np.delete(tskip, i)
        try:
            paramsi = curve_fit(func, ttest, ytest, p0=p0, maxfev=10000)[0]
            yfiti = func(tskip[i], *paramsi)
            erri = (yskip[i]-yfiti)**2.
        except RuntimeError:
            erri = 1.e3
        cv = cv + erri
    # Average cv scores
    cv = cv/np.float(n)

    # If fit is not found for this window, penalize strongly
    try:
        paramsi = curve_fit(func, t, y, p0=p0, maxfev=10000)[0]
    except RuntimeError:
        cv = 1.e6
    return cv


def minimize_cv_error(t, y, twindows, func, p0=None):
    """ Find the fitting interval that minimizes the cross-validation
    error for a model fitted to a sub-interval of a dataset, 
    given a set of possible intervals in the independent variable 
    over which to perform the fit.

    Parameters
    ----------
    t : 1-d array
        Length N vector  of values for the independent variable of a dataset.
    y : 1-d array
        Length N vector of values for the dependent variable of a dataset.
    twindows : List of len 2 lists
               List of the form [[t0, tend1], [t0, tend2], ...]
               where [t0, tendi] represents the `ith` closed interval over which to
               fit the data set and find the cross-validation score.
    func : callable function
           Model to fit the data against. Must be of the form
           `f(t, p1, p2, ..., pM)` where `t` is the independent variable and
           `p1, p2, ..., pM` is a set of M parameters to fit.

    p0 : 1-d array or list, `optional`
         Initial guesses for the M parameters to fit, [p1, p2, ..., pM]

    Returns
    -------
    twindow_min : List
                  List of the form `[t0, tend]' where t0 and tend are the
                  beginning and end of the closed interval in ``t`` that
                  provides the lowest cross-validation score for the fitted
                  model
    pmin : list
           Optimal parameters for fitting the model ``func`` to the data
           over the sub-interval ``twindow_min``
    CV_min : float
             Leave-one-out cross-validation error for the model ``func``
             over the interval ``twindow_min``
    """
    CVs = []
    params = []
    for twindow in twindows:
        tinds = [np.argmin(np.abs(t-twindow[0])),
                 np.argmin(np.abs(t-twindow[1]))]
        tfit = t[tinds[0]: tinds[1]+1]
        yfit = y[tinds[0]: tinds[1]+1]
        CVs.append(get_cross_validation_score(tfit, yfit, func, p0))
        try:
            paramsi = curve_fit(func, tfit, yfit, p0=p0, maxfev=100000)[0]
            params.append(paramsi)
        except RuntimeError:
            params.append(None)
    CV_argmin = np.argmin(CVs)
    CV_min = CVs[CV_argmin]
    twindow_min = twindows[CV_argmin]
    pmin = params[CV_argmin]
    return [twindow_min, pmin, CV_min]


def laplace_merge(omega, G1, G2, G1_Fdirect, G2_Fdirect):
    """ Find the interval where the Laplace transform is valid over
    the frequency space and replace the Fourier transform data with 
    the Laplace transform modulus.

    Parameters
    ----------
    omega : 1-d array
            Vector of angular frequencies in units of rad/s
    G1 : 1-d array
         Storage modulus at angular frequencies ``omega`` in units of Pa
         calculated using Fourier transform
    G2 : 1-d array
         Loss modulus at angular frequencies ``omega`` in units of Pa
         calculated using Fourier transform
    G1_Fdirect: 1-d array
                Storage modulus at angular frequencies ``omega`` in units 
                of Pa calculated using direct Laplace transform
    G2_Fdirect: 1-d array
                Loss modulus at angular frequencies ``omega`` in units
                of Pa calculated using direct Laplace transform

    Returns
    -------
    G1_plot : 1-d array
              Loss modulus at angular frequencies ``omega`` in units of Pa
              after merging Laplace and Fourier transform moduli
    G2_plot : 1-d array
              Storage  modulus at angular frequencies ``omega`` in units of
              Pa after merging Laplace and Fourier transform moduli
    """
    G1_lower = 0
    G2_lower = 0
    G1_upper = len(omega) - 1
    G2_upper = len(omega) - 1
    for n in range(len(omega)-1):
        if (G1[n]-G1_Fdirect[n])/np.abs(G1[n]-G1_Fdirect[n]) != (G1[n+1]-G1_Fdirect[n+1])/np.abs(G1[n+1]-G1_Fdirect[n+1]):
            G1_lower = n
            break
    for n in range(len(omega)-1):
        if (G2[n]-G2_Fdirect[n])/np.abs(G2[n]-G2_Fdirect[n]) != (G2[n+1]-G2_Fdirect[n+1])/np.abs(G2[n+1]-G2_Fdirect[n+1]):
            G2_lower = n
            break
    for n in range(len(omega)-1):
        if (G1[len(omega)-1-n]-G1_Fdirect[len(omega)-1-n])/np.abs(G1[len(omega)-1-n]-G1_Fdirect[len(omega)-1-n]) != (G1[len(omega)-2-n]-G1_Fdirect[len(omega)-2-n])/np.abs(G1[len(omega)-2-n]-G1_Fdirect[len(omega)-2-n]):
            G1_upper = len(omega)-1-n
            break
    for n in range(len(omega)-1):
        if (G2[len(omega)-1-n]-G2_Fdirect[len(omega)-1-n])/np.abs(G2[len(omega)-1-n]-G2_Fdirect[len(omega)-1-n]) != (G2[len(omega)-2-n]-G2_Fdirect[len(omega)-2-n])/np.abs(G2[len(omega)-2-n]-G2_Fdirect[len(omega)-2-n]):
            G2_upper = len(omega)-1-n
            break

    G1_plot = np.zeros_like(G1)
    G2_plot = np.zeros_like(G2)
    G1_plot[0:G1_lower] = G1[0:G1_lower]
    G1_plot[G1_lower:G1_upper] = G1_Fdirect[G1_lower:G1_upper]
    G1_plot[G1_upper:] = G1[G1_upper:]
    G2_plot[0:G2_lower] = G2[0:G2_lower]
    G2_plot[G2_lower:G2_upper] = G2_Fdirect[G2_lower:G2_upper]
    G2_plot[G2_upper:] = G2[G2_upper:]
    return G1_plot, G2_plot





import numpy as np
from dlsmicro.backend import plot_tools
import matplotlib.pyplot as plt
import pandas as pd
from matplotlib import rc
import matplotlib as mpl

def plot_conditions(df_path, condition_dir, replicate_dict, cond_color=None, 
					cond_label=None, plot_ci=True, plot_G_replicates=True, 
					plot_alpha_replicates=True, plot_scattering=False, 
					add_scaling=False, scaling_frac=None):

	""" Plot DLS microrheology output data for
    multiple conditions after analysis and saving as dataframe.

    Parameters
    ----------
    df_path : str
    		  Path to saved dataframe to plot
    condition_dir : dictionary
    			    Dictionary of conditions and respective folders
    cond_color : dictionary, `optional`
    			 Dictionary of conditions and respective color
    			 If `None`, colors are default colors
    cond_label : dictionary, `optional`
    			 Dictionary of conditions and respective labels
    			 If `None`, labels are taken to be folder name
    plot_ci : boolean, `optional`
    		  If `True`, plot error bars around G and alpha
    plot_G_replicates : boolean, `optional`
    					If `True`, plot G for all conditions averaged
    					over all replicates
    plot_alpha_replicates : boolean, `optional`
    						If `True`, plot alpha for all conditions
    						averaged over all replicates
    plot_scattering : boolean, `optional`
    				  If `True`, plot scattering for all conditions
    add_scaling : boolean, `optional`
    			  If `True`, add scaling defined by `scaling_frac` to plot
    scaling_frac : list of float, `optional`
    			   List of 2 floats, where the first number is numerator 
    			   of fraction and second is denominator of fraction
    """
	
	conditions = list(condition_dir.keys())
	df = pd.read_pickle(df_path)

	# set colors and labels in dictionaries
	if cond_color == None:
		cmap = plt.cm.get_cmap('hot',len(conditions)*2)
		colors = [cmap(i) for i in range(len(conditions))]
		cond_color = dict(zip(conditions, colors))
	if cond_label == None:
		cond_label = condition_dir 

	if plot_G_replicates:
		# Plot style options
		rc('axes', labelsize=24.)
		rc('xtick', labelsize=18.)
		rc('ytick', labelsize=18.)
		rc('lines', markersize=10)
		rc('lines', linewidth=3.)

		fig, ax1 = plt.subplots(1, 1)
		dash_style = (6, 2)
		for condition in conditions:
			dfi = df[df['condition'] == condition]
			plot_tools.plot_replicates_from_df(dfi, 'G1', plot_ci=plot_ci, 
											   color=cond_color[condition])
			plot_tools.plot_replicates_from_df(dfi, 'G2', plot_ci=plot_ci, 
				                               color=cond_color[condition], ls='--')

		if add_scaling:
			plot_tools.add_w_scaling(dfi[dfi['replicate'] == replicate_dict[conditions[0]][0]]['omega'][0:50],
						  scaling_frac,
						  dfi[dfi['replicate'] == replicate_dict[conditions[0]][0]]['G2'][2],
						  [2./6.,3./6.])

		# Define line objects for legend so that line color is black rather than
		# inheriting the color of one of the conditions
		handles = [None] * (len(conditions)+2)
		for con in range(len(conditions)):
			handles[con+2] = plt.Line2D((0, 1), (0, 0), color=cond_color[conditions[con]], 
										label=cond_label[conditions[con]])
		handles[0] = plt.Line2D((0, 1), (0, 0), color='k', dashes='',
		            label='$\mathregular{G^{\prime}}$')
		handles[1] = plt.Line2D((0, 1), (0, 0), color='k', ls='--',
		            label='$\mathregular{G^{\prime \prime}}$')
		plt.legend(handles=handles, frameon=False, fontsize=10, handlelength=2.0)
		plt.ylabel('$\\mathregular{G^*\ (Pa)}$')
		plt.xlabel('$\\mathregular{\\omega\ (s^{-1})}$')
		plt.xscale('log')
		plt.yscale('log')

		# Force plot to show ticks at every decade and minor ticks
		minor_tick_marks = [0.1*x for x in range(1, 10)]
		locmax = mpl.ticker.LogLocator(base=10, numticks=12)
		locmin = mpl.ticker.LogLocator(base=10, numticks=12, subs=minor_tick_marks)
		ax1.xaxis.set_major_locator(locmax)
		ax1.xaxis.set_minor_locator(locmin)
		ax1.xaxis.set_minor_formatter(mpl.ticker.NullFormatter())
		plt.tight_layout()
		plt.show()

	if plot_alpha_replicates:
		# Plot style options
		rc('axes', labelsize=24.)
		rc('xtick', labelsize=18.)
		rc('ytick', labelsize=18.)
		rc('lines', markersize=10)
		rc('lines', linewidth=3)
		for condition in conditions:
			dfi = df[df['condition'] == condition]
			plot_tools.plot_replicates_from_df(dfi, 'alpha', plot_ci=plot_ci, 
											   color=cond_color[condition])

		y_matrix = plot_tools.df_to_matrix(df[df['condition'] == 'condition1'], 
			                               'alpha','replicate')
		replicates = df['replicate'].values
		time = df[df['replicate'] == replicates[0]]['omega'].values
		plt.plot(time,np.full(np.shape(time),2./3.),'k--')

		# Define line objects for legend so that line color is black rather than
		# inheriting the color of one of the conditions
		handles = [None] * len(conditions)
		for con in range(len(conditions)):
			handles[con] = plt.Line2D((0, 1), (0, 0), color=cond_color[conditions[con]], 
									  label=cond_label[conditions[con]])
		plt.legend(handles=handles, frameon=False, fontsize=10, handlelength=2.0)
		plt.ylabel('$\\mathregular{\\alpha}$')
		plt.xlabel('$\\mathregular{\\omega\ (s^{-1})}$')
		plt.xscale('log')
		plt.tight_layout()
		plt.show()

	if plot_scattering:
		# Plot style options
		rc('axes', labelsize=14.)
		rc('xtick', labelsize=14.)
		rc('ytick', labelsize=14.)
		rc('lines', markersize=10)
		rc('lines', linewidth=3)

		for condition in conditions:
			dfi = df[df['condition'] == condition]
			ids = set(dfi['replicate'].values)
			for idx in ids:
				epos = dfi[dfi['replicate'] == idx]['epos'].values
				scattering = dfi[dfi['replicate'] == idx]['scattering'].values
				plot_over = np.argwhere(epos)
				plt.plot(epos[plot_over],scattering[plot_over],color=cond_color[condition],
					     linestyle='None',marker='.')

		# Define line objects for legend so that line color is black rather than
		# inheriting the color of one of the conditions
		handles = [None] * len(conditions)
		for con in range(len(conditions)):
			handles[con] = plt.Line2D((0, 1), (0, 0), color=cond_color[conditions[con]],
		                       ls='None', marker='o', label=cond_label[conditions[con]])
		plt.legend(handles=handles, frameon=False, fontsize=10, handlelength=2.0)
		plt.ylabel('Scattering Intensity')
		plt.xlabel('Position')
		plt.yscale('log')
		plt.tight_layout()
		plt.show()



""" Module for processing DLS correlation data into rheological properties"""

import numpy as np
import dlsmicro.backend.utils as utils
from scipy import special
import dlsmicro.backend.fit_funcs as fit_funcs
import pandas as pd


def find_g0(t, corr, func=fit_funcs.stretched_exp, t0=2.0,
            tmaxs=np.arange(40., 130., 10), p0=None):
    """ Estimate the intercept of the correlation function at t = 0

    Parameters
    ----------
    t  : 1d-array
         Vector of time-lags at which the correlation coefficient is computed
    corr : 1d-array
         Correlation coefficient (which spans possible from 0.0 to 1.0).
         Note that this is equal to `g2 - 1`, where `g2` is the intensity
         autocorrelation function.
    func : callable function
           Model to fit the correlation against. Must be of the form
           `f(t, p1, p2, ..., pM)` where `t` is the independent variable and
           `p1, p2, ..., pM` is a set of M parameters to fit.
    t0: float, `optional`
        Minimum time-lag to use in the fit of the correlation coefficient
        for estimating the intercept.
    tmaxs : 1-d array or list of floats, `optional`
           Vector of possible maximum time-lags to use in the fitting interval
           of the correlation coefficient. The optimal tmax will be selected
           based on minimization of the cross-validation error of the fit.
    p0 : 1-d array, `optional`
         Initial guesses for the parameters to ``func`` for fitting


    Returns
    -------
    g0 : float
         Estimate of the intercept of ``corr`` at `t=0`
    twindow_min : List of floats
                  Optimized interval in time over which the fit to ``corr``
                  was performed to obtain g0. This is a list
                  of the form [t0, tmax].
    pmin : 1d-array
           Optimal values for the fitting parameters to ``func`` used to
           estimate ``g0``
    """

    # Construct list of fitting windows
    twindows = [[t0, tmax] for tmax in tmaxs]
    # Find twindow for minimum CV error
    [twindow_min, pmin, CV_min] = utils.minimize_cv_error(t, corr, twindows,
                                                          func, p0)
    g0 = func(0.0, *pmin)

    return [g0, twindow_min, pmin]


def calc_g1(t, corr, ergodic, g0=None, Ip=None, Ie=None, eps=None):
    """Compute the intermediate scattering function from the correlation function.

    This function transforms the correlation functio to the intermediate 
    scattering function, which is necessary for computing the particle MSD.
    The function enables corrections for broken-ergodicty and estimation
    of the zero-time intercept of the correlation function.

    Parameters
    ----------
    t  : 1d-array
         Vector of time-lags at which the correlation function is computed
    corr : 1d-array
         Correlation coefficient (which spans possible from 0.0 to 1.0).
         Note that this is equal to `g2 - 1`, where `g2` is the intensity
         autocorrelation function.
    ergodic: boolean
             If False, corrections are made for non-ergodic systems
             based on ``Ip`` and ``Ie``
    Ip : float
        Scattering intensity at the position
        where correlation function is collected.
        (Only used if ``ergodic=False``)
    Ie : 1d-array
        Vector of scattering intensities at different positions in the cuvette
        (Only used if ``ergodic=False``)
    g0 : float, `optional`
         Estimate of the intercept of ``g2 - 1`` at time 0. If not provided,
         the intercept will be estimated automatically based on a stretched
         exponential fit.

    Returns
    -------
    g1 : 1d-array
        The intermediate scattering function corresponding to
        the time-lags ``t``

    Notes
    -----
    The correlation function exported by the Zetasizer software is equal to
    ``g2 - 1``
    """

    g2 = corr + 1.

    # If no intercept is provided, estimate it using a stretched exponential
    # function with default parameters

    if g0 is None:
        # Define guesses for the stretched exponential function fitting
        a0 = 1.0e-2
        beta0 = 1.0
        p0 = [corr[1], a0, beta0]
        # Fit the correlation and get the intercept
        [g0, twindow_min, pmin] = find_g0(t, corr, func=fit_funcs.stretched_exp, t0=2.0,
                     tmaxs=np.arange(40., 130., 10.), p0=p0)
    
    # If any of the g values are greater than g0, allow
    # g2 to be replaced with the fit to avoid negative
    # MSD values
    if np.any(corr > g0):
        print('Negative values encountered in MSD...')
        print('Replacing early time data with gfit')
    tmin = np.argmin(np.abs(t-twindow_min[0]))
    tmax = np.argmin(np.abs(t-twindow_min[1]))
    gfit = fit_funcs.stretched_exp(t, *pmin)
    g2[tmin:tmax] = gfit[tmin:tmax] + 1.

    if ergodic:
        g1 = np.sqrt((g2-1.)/g0)
    else:
        Ie_avg = np.average(Ie)
        # calculate the ratio of ensemble to time averaged
        # intensities
        Y = Ie_avg/Ip
        # calculate the g1 correlation function
        if eps is None:
            g1 = (Y-1.)/Y+np.sqrt(g2-g0)/Y
        else:
            g1 = (1.-(1.-eps)/Y +
                  (1-eps)*np.sqrt(1. +
                                  (g2-g0-1.)/(1-eps)**2.)/Y)
    return g1


def calc_q(n, theta, lam):
    """ Calculate the scattering wave-vector.

    This function returns the scattering wave-vector for a DLS experiment
    based on the scattering geometry and the wave-length of the laser.

    Parameters
    ----------
    n : float
        Index of refraction of the material (1.333 for water)
    theta : float
            Scattering angle in radians
    lam : float
          wave-length of laser (units used here determine the
          units of ``q``)

    Returns
    -------
    q : float
        Scattering wave-vector in inverse units of ``1/lam``
    """
    q = 4.*np.pi*n*np.sin(theta/2.)/(lam)
    return q


def msd_local_pwr_law(t, g1, q, bw=0.1, replace_neg=True):
    """ Calculate the local power-law scaling of the MSD and the
        smoothed MSD by locally-weighted logarithmic linear regression

    Parameters
    ----------
    t : 1d-array
        Vector of time-lags
    g1 : 1d-array
         Vector containing the intermediate scattering function at time-lags
         given by ``t``
    q : float
        Scattering vector in units of 1/nm
    bw : float, `optional`
           Bandwith smoothing parameter for locally-weighted regression.
           Reasonable values are typically between 0.05 and 0.1


    Returns
    -------
    msd_smooth: 1d-array
                Vector of smoothed MSD values from the local regression
                corresponding to the time lags in ``t`` (in units of nm^2).
    alpha: 1d-array
           Vector of local power-law scaling exponents of the MSD
           corresponding to the time lags in ``t``.
    """
    msd = -6*np.log(g1)/(q**2.)

    # Remove data points with 0, negative, or infinite MSD
    if replace_neg:
        if any(msd < 0):
            print('negative msd values enountered,'
                  'replacing with interpolation')
            neg_inds = msd < 0
            t_pos = t[msd > 0]
            msd_pos = msd[msd > 0]
            t_neg = t[neg_inds]
            msd_interp = np.interp(t_neg, t_pos, msd_pos)
            msd[neg_inds] = msd_interp

    # Perform a locally-weighted logarithmic linear regression
    [Theta, log_msd_smooth] = utils.loess(np.log(t), np.log(msd),
                                          degree=1, alpha=bw)
    msd_smooth = np.exp(log_msd_smooth)
    # Define the local power law scaling exponent based
    # on the logarithmic slopes
    alpha = Theta[1, :]
    return [msd_smooth, alpha]


def calc_msd_raw(t, g1, q, replace_neg=True):
    """ Calculate the MSD from the intermediate scattering function.

    Parameters
    ----------
    t : 1d-array
        Vector of time-lags
    g1 : 1d-array
         Vector containing the intermediate scattering function at time-lags
         ``t``
    q : float
        Scattering vector in units of 1/nm
    replace_neg : boolean, `optional`
                  If ``True``, replace negative values of the MSD by linear
                  interpolation

    Returns
    -------
    msd : 1d-array
          Vector of mean-squared-displacements at time-lags ``t``
          (in units of nm^2)
    """
    msd = 6*np.log(g1)/(q**2.)

    # Remove data points with 0, negative, or infinite MSD
    if replace_neg:
        if any(msd < 0):
            print('negative msd values enountered,'
                  'replacing with interpolation')
            neg_inds = msd < 0
            t_pos = t[msd > 0]
            msd_pos = msd[msd > 0]
            t_neg = t[neg_inds]
            msd_interp = np.interp(t_neg, t_pos, msd_pos)
            msd[neg_inds] = msd_interp

    return msd


def shear_modulus(t, msd, alpha, r, T):
    """ Calculate Frequency-dependent shear modulus by power-law analysis 

    Calculate the frequency-dependent complex shear modulus using results
    from a power-law analysis of the mean-squared-displacement. 

    Parameters
    ----------
    t : 1d-array
        Time-lags in units of microseconds
    msd : 1d-array
          Mean-squared displacements at the time-lags
          ``t`` in units of nm^2
    alpha : 1d-array of local power-law scaling exponents
    r : float
        Radius of the probe particles in nanometers
    T : float
        Temperature in Kelvin

    Returns
    -------
    omega : 1d-array
            Vector of angular frequencies in units of 1/s
    G1 : 1d-array
         Storage modulus at angular frequencies ``omega`` in
         units of Pa
    G2 : 1d-array
         Loss modulus at angular frequencies ``omega`` in
         units of Pa

    Notes
    -----
    This method produces more accurate results in the upper and
    lower frequency extremes than a direct numerical Laplace
    or Fourier transform.
    """
    # Boltzman constant
    kb = 1.38e-23
    # magnitude of the modulus
    G = (msd**-1.)*1./(np.pi*r*special.gamma(1.+alpha))
    G = kb*T*(1.e27)*G
    # Storage G1 and loss G2 moduli
    G1 = G*np.cos(np.pi*alpha/2.)
    G2 = G*np.sin(np.pi*alpha/2.)
    # Calculate omega
    omega = t**-1.
    # Convert omega to 1/s
    omega = omega*(1.e6)
    return [omega, G1, G2]


def shear_modulus_laplace_transform(t, msd, r, T, bw=0.01):
    """ Calculate the shear modulus by direct laplace transform of the MSD

    Parameters
    ----------
    t : 1d-array
        Time-lags in units of microseconds
    msd : 1d-array
          Mean-squared displacements at the time-lags
          ``t`` in units of nm^2
    r : float
        Radius of the probe particles in nanometers
    T : float
        Temperature in Kelvin
    bw : float, optional
         Bandwith parameter for analytic continuation of
         the laplace transform into fourier space by locally weighted
         regression

    Returns
    -------
    omega : 1d-array
            Vector of angular frequencies in units of 1/s
    G1 : 1d-array
         Storage modulus at angular frequencies ``omega`` in
         units of Pa
    G2 : 1d-array
         Loss modulus at angular frequencies ``omega`` in
         units of Pa

    Notes
    -----
    This function returns the shear modulus in the Fourier (omega) domain
    by first transforming to the Laplace (s) domain and then performing
    analytic continuation onto the imaginary axis in the complex plane.

    This method produces erroneous results in the upper and lower frequency
    extremes (roughly the first and last decade of frequencies).
    """
    # Boltzman constant
    kb = 1.38e-23
    # Calculate the direct laplace transform of the mean-squared displacement
    s = t**-1.
    msd_laplace = utils.laplace(t, msd, s)
    # Calculate the G*s in laplace space
    Gs = (msd_laplace**-1.)/(np.pi*r*s)
    # Perform a local power-law analysis of the laplace space shear modulus
    [Theta, logGs] = utils.loess(np.log(s), np.log(Gs), degree=1, alpha=bw)
    # Calculate the local scaling exponent
    alpha_direct = Theta[1, :]
    G = np.exp(logGs)
    G = kb*T*(1.e27)*G
    # Storage G1 and loss G2 moduli
    G1 = G*np.cos(np.pi*alpha_direct/2.)
    G2 = G*np.sin(np.pi*alpha_direct/2.)
    # Calculate omega
    omega = t**-1.
    # Convert omega to 1/s
    omega = omega*(1.e6)
    return [omega, G1, G2]


def full_dlsur_analysis(t, corr, ergodic, r, T, q, Ip, Ie,
                        calc_g1_kws={}, pwr_law_kws={}):
    """ Perform a full microrheology analysis from the correlation function.

    This function returns a table reporting particle motion statistics
    and the material shear modulus, beginning with the correlation
    coefficient. It implicitly calculates the intercept of the correlation
    function and a power-law analysis of the MSD to perform an algebraic
    calculation of the shear modulus. 

    Parameters
    ----------

    t : 1d-array
        Vector of lag-times (in microseconds)
    corr : 1d-array
           Vector of values of the correlation coefficient, corresponding
           to the time-lags ``t``.
    ergodic : boolean
              If ``ergodic==False`` then corrections are applied to the
              calculation of `g1`, based on ``Ip`` and ``Ie``
    r : float
        Particle radius in nanometers
    T : float
        Temperature in Kelvin
    q : float
        Scattering vector in 1/nm
    Ip : float, `optional`
         Scattering intensity at the meausurement position where ``corr``
         is collected
    Ie : 1d-array
        Vector of scattering intensities at different measurement positions
        in the cuvette
    calc_g1_kws : dictionary, `optional`
                  Dictionary of keyword arguments to pass to
                  ``analysis.tools.calc_g1()`` for computing the
                  intermediate scattering function
    pwr_law_kws : dictionary, `optional`
                  Dictionary of keyword arguments to pass to
                  ``analysis_tools.msd_local_pwr_law()`` for local
                  power-law analysis of the msd

    Returns
    -------
    dlsmicro_df : DataFrame
                  Dataframe containing table of results from DLS microrheology
                  analysis
    """

    # Find the intermediate scattering function
    g1 = calc_g1(t, corr, ergodic, Ip=Ip, Ie=Ie, **calc_g1_kws)

    # Calculate the power-law smoothing of the msd
    [msd_smooth, alpha] = msd_local_pwr_law(t, g1, q,
                                            **pwr_law_kws)

    # Calculate the shear modulus from the power-law smoothing
    [omega, G1, G2] = shear_modulus(t, msd_smooth, alpha, r, T)

    dlsmicro_dict = {'t': t, 'msd_smooth': msd_smooth,
                     'alpha': alpha, 'omega': omega,
                     'G1': G1, 'G2': G2}
    dlsmicro_df = pd.DataFrame(dlsmicro_dict)

    return dlsmicro_df



import numpy as np
import matplotlib.pyplot as plt

def df_to_matrix(df, quantity, replicate_identifier):

	""" Construct a matrix from a Dataframe for a given vector-valued 
	quantity in which each vector replicate is labeled by a unique 
	replicate identifier.

    Parameters
    ----------
    df : DataFrame
         Dataframe containing table of results from DLS microrheology
         analysis for a single condition
    quantity : str
          	   Name of variable to plot as defined in the Dataframe
   	replicate_identifier : str
   				 		   Name of quantity to average over

   	Returns
    -------
    M : 2-d array
        Matrix where each row is a replicate of the vector quantity.
    """

    # Construct a matrix in which each row represents a frequency sweep
	ids = set(df[replicate_identifier].values)
	M_list = []
	list_len = []
	for idx in ids:
	    xi = df[df[replicate_identifier] == idx][quantity].values
	    list_len.append(len(xi))
	    M_list.append(xi)
    # Ensure all rows are the same length
	limit = np.min(np.array(list_len))
	M_list = list(m[0:limit] for m in M_list)
	M = np.vstack(M_list)
	return M

def bootstrap_matrix_byrows(M, n_bootstrap, estimator):

	""" Gets bootstrap samples of an estimator for frequency (or time)
	sweep data from a matrix containing all vectors for a given quantity
	over all replicates.

    Parameters
    ----------
    M : 2-d array
        Matrix of all values for a given quantity over all replicates
   	n_bootstrap : int
   				  Number of points for bootstrap
    estimator : callable function
    			Function for evaluating center of distribution

   	Returns
    -------
    M_bootstrap : 2-d array
        	 	  Matrix in which each row represents a frequency sweep
    """
	n_rep = M.shape[0]
	M_bootstrap = np.zeros((n_bootstrap, M.shape[1]))
	M_sample = np.zeros(M.shape)
	for i in range(n_bootstrap):
	    inds = np.random.randint(0, n_rep, n_rep)
	    M_sample = M[inds, :]
	    M_bootstrap[i, :] = estimator(M_sample, axis=0)
	return M_bootstrap

def bootstrap_freq_sweep(df, quantity, replicate_identifier,
                         n_bootstrap, estimator=np.mean):

	""" Gets bootstrap samples of an estimator for frequency (or time)
	sweep data from a Dataframe. The Dataframe is assumed to contain a 
	vector for a given quantity in which each vector replicate is 
	labeled by a unique replicate identifier.

    Parameters
    ----------
    df : DataFrame
         Dataframe containing table of results from DLS microrheology
         analysis for a single condition
    quantity : str
          	   Name of variable to plot as defined in the Dataframe
   	replicate_identifier : str
   				 		   Name of quantity to average over
   	n_bootstrap : int
   				  Number of points for bootstrap
    estimator : callable function
    			Function for evaluating center of distribution

   	Returns
    -------
    M_bootstrap : 2-d array
        	 	  Matrix in which each row represents a frequency sweep
    """

	ids = set(df[replicate_identifier].values)
	M_list = []
	list_len = []
	for idx in ids:
	    xi = df[df[replicate_identifier] == idx][quantity].values
	    list_len.append(len(xi))
	    M_list.append(xi) 
	limit = np.min(np.array(list_len))
	M_list = list(m[0:limit] for m in M_list)
	M = np.vstack(M_list)

    # Get a matrix of bootstrapped row-wise averages given by the estimator
	M_bootstrap = bootstrap_matrix_byrows(M, n_bootstrap, estimator)

	return M_bootstrap

def bootstrap_freq_sweep_ci(df, quantity, replicate_identifier,
                            n_bootstrap, ci, estimator=np.mean):

	""" Gets bootstrap confidence interval for an estimator of
	frequency sweep (or time sweep) data. Boot strap can be either a 
	percentile bootstrap of an estimator of a studentized bootstrap
	of the mean

    Parameters
    ----------
    df : DataFrame
         Dataframe containing table of results from DLS microrheology
         analysis for a single condition
    quantity : str
          	   Name of variable to plot as defined in the Dataframe
   	replicate_identifier : str
   				 		   Name of quantity to average over
   	n_bootstrap : int
   				  Number of points for bootstrap
   	ci : float
   		 Percent of distribution included in error bars
    estimator : callable function, `optional`
    			Function for evaluating center of distribution
    
   	Returns
    -------
    ci_low : 1-d array
        	 Vector of the lower bound of the confidence interval over 
        	 entire frequency range.
    ci_high : 1-d array
			  Vector of the upper bound of the confidence interval
			  over entire frequency range.
    """

	M_bootstrap = bootstrap_freq_sweep(df, quantity, replicate_identifier,
                                       n_bootstrap, estimator=estimator)
	ci_low = np.percentile(M_bootstrap, 50.-ci/2., axis=0)
	ci_high = np.percentile(M_bootstrap, 50.+ci/2., axis=0)
	return [ci_low, ci_high]

def plot_replicates_from_df(df, my_quantity, plot_ci=True, myci=68., 
							estimator=np.mean, color='m', ls='-', 
							err_alpha=0.25, err_lw=2.5, identifier='replicate'):

    """ Plot a given quantity from the Dataframe, averaging across
    all replicates in that Dataframe.

    Parameters
    ----------
    df : DataFrame
         Dataframe containing table of results from DLS microrheology
         analysis for a single condition
    my_quantity : str
          		  Name of variable to plot as defined in the Dataframe
    plot_ci : boolean, `optional`
        	  If `True`, plot error bars
    myci : float, `optional`
    	   Percent of distribution plotted in error bars 
   	estimator : callable function, `optional`
   				Function for evaluating main plotted value
   	color : str, `optional`
   			Color of plotted line
   	ls : str, `optional`
   		 Linestyle of plotted line
   	err_alpha : float, `optional`
   				Transparency level of error bars
   	err_lw : float, `optional`
   			 Linewidth of error bar outlines
   	identifier : str, `optional`
   				 Name of quantity to average over
    """

    y_matrix = df_to_matrix(df, my_quantity, identifier)
    replicates = df['replicate'].values
    time = df[df['replicate'] == replicates[0]]['omega'].values[0:np.shape(y_matrix)[1]]
    ci = bootstrap_freq_sweep_ci(df, my_quantity, identifier, 10000,
    	                         myci, estimator=estimator)
    ci_low = ci[0]
    ci_high = ci[1]
    y_mu = estimator(y_matrix, axis=0)
    plt.plot(time, y_mu, color=color, ls=ls)
    time = np.array(time, dtype=float)
    ci_low = np.array(ci_low, dtype=float)
    ci_high = np.array(ci_high, dtype=float)
    if plot_ci:
        plt.fill_between(time, ci_low, ci_high, color=color,
                         alpha=err_alpha, linewidth=err_lw)

def add_w_scaling(omega, scaling, w_b, placement):
    
  """ Plot a given scaling on complex modulus plot to compared against the complex modulus of a sample.

    Parameters
    ----------
    omega : 1-d array
    		Vector of frequency range covered by the complex modulus
    		plotted in the plot
    scaling : list of float
          	  List of 2 floats, where the first number is numerator 
          	  of fraction and second is denominator of fraction
    w_b : float
          Value of complex modulus plotted where scaling should appear
          on the plot
    placement : list of float
    	   		First element in list is lower bound of scaling line,
    	   		second element in list is upper bound of scaling line,
    	   		where both elements are values between 0 and 1. The 
    	   		value of the first element should be less than the value
    	   		of the second element
    """

  lolim = np.int(len(omega)*placement[0])
  hilim = np.int(len(omega)*placement[1])
  omega = np.array(omega,dtype=float)
  g_scale = np.float_power(omega, scaling[0]/scaling[1])*w_b/np.float_power(omega[lolim],scaling[0]/scaling[1])
  plt.plot(omega[lolim:hilim], g_scale[lolim:hilim], ls='--', color='k',linewidth=2)
  model = np.int(0.6*(lolim+hilim))
  plt.text(omega[model],g_scale[model]*1.6,
  	     '$\omega^{%(top)s/%(bot)s}$'%{'top':np.int(scaling[0]),
  	     'bot':np.int(scaling[1])},fontsize=12)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import importlib
import csv
import os
import sys
from setuptools import setup, find_packages
import io
import versioneer

# Constants
DISTNAME = 'DLSuR'
DESCRIPTION = 'Dynamic light scattering microrheology data analysis package'
URL = 'https://github.com/PamCai/DLSuR'
LICENSE = 'BSD-3-Clause'


# Parse description
with io.open('README.md', encoding='utf8') as f:
    README = f.read().split('\n')
    LONG_DESCRIPTION = '\n'.join([x for x in README if not x[:3]=='[!['])


VERSION = versioneer.get_version()
CMDCLASS = versioneer.get_cmdclass()


# Parse requirements.txt
with io.open('requirements.txt', 'r') as f:
    REQUIREMENTS = f.read().split('\n')


setup_info = dict(
    # Metadata
    name=DISTNAME,
    version=VERSION,
    author=('Brad Krajina, Pam Cai,'),
    author_email=('bradkrajina@gmail.com, pamela.5.cai@gmail.com, '),
    url=URL,
    classifiers=['Intended Audience :: Education',
                 'Intended Audience :: Science/Research',
                 'License :: OSI Approved :: BSD License',
                 'Natural Language :: English',
                 'Operating System :: MacOS',
                 'Operating System :: POSIX :: Linux',
                 'Programming Language :: Python :: 3.5',
                 'Programming Language :: Python :: 3.6',
                 'Programming Language :: Python :: 3.7',
                 'Programming Language :: Python :: 3.8',
                 'Programming Language :: Python :: 3.9',
                 'Topic :: Scientific/Engineering :: Information Analysis',
                 'Topic :: Scientific/Engineering :: Mathematics',
                 'Topic :: Scientific/Engineering :: Physics',
                 ],
    cmdclass=CMDCLASS,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    python_requires='>=3.5',
    license=LICENSE,
    packages=find_packages(exclude=('test',)),
    install_requires=REQUIREMENTS,
    zip_safe=True,
)

setup(**setup_info)